import { __assign, __awaiter, __generator, __makeTemplateObject, __spreadArrays } from "tslib";
import * as changeCase from 'change-case';
import fs from 'fs-extra';
import got from 'got';
import path from 'path';
import prettier from 'prettier';
import { castArray, dedent, groupBy, isEmpty, isFunction, memoize, omit, uniq, values, } from 'vtils';
import { getRequestDataJsonSchema, getResponseDataJsonSchema, jsonSchemaToType, throwError, } from './utils';
var Generator = /** @class */ (function () {
    function Generator(config, options) {
        var _this = this;
        if (options === void 0) { options = { cwd: process.cwd() }; }
        this.options = options;
        /** 配置 */
        this.config = [];
        this.disposes = [];
        this.fetchProject = memoize(function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return __awaiter(_this, void 0, void 0, function () {
                var projectInfo, basePath;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, this.fetchApi(serverUrl + "/api/project/get", {
                                token: token,
                            })];
                        case 1:
                            projectInfo = _b.sent();
                            basePath = ("/" + (projectInfo.basepath || '/'))
                                .replace(/\/+$/, '')
                                .replace(/^\/+/, '/');
                            projectInfo.basepath = basePath;
                            return [2 /*return*/, projectInfo];
                    }
                });
            });
        }, function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return serverUrl + "|" + token;
        });
        this.fetchExport = memoize(function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return __awaiter(_this, void 0, void 0, function () {
                var projectInfo, categoryList;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0: return [4 /*yield*/, this.fetchProject({ serverUrl: serverUrl, token: token })];
                        case 1:
                            projectInfo = _b.sent();
                            return [4 /*yield*/, this.fetchApi(serverUrl + "/api/plugin/export", {
                                    type: 'json',
                                    status: 'all',
                                    isWiki: 'false',
                                    token: token,
                                })];
                        case 2:
                            categoryList = _b.sent();
                            return [2 /*return*/, categoryList.map(function (cat) {
                                    cat.list = (cat.list || []).map(function (item) {
                                        item.path = "" + projectInfo.basepath + item.path;
                                        return item;
                                    });
                                    return cat;
                                })];
                    }
                });
            });
        }, function (_a) {
            var serverUrl = _a.serverUrl, token = _a.token;
            return serverUrl + "|" + token;
        });
        // config 可能是对象或数组，统一为数组
        this.config = castArray(config);
    }
    Generator.prototype.prepare = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this;
                        return [4 /*yield*/, Promise.all(
                            // config 可能是对象或数组，统一为数组
                            this.config.map(function (item) { return __awaiter(_this, void 0, void 0, function () {
                                return __generator(this, function (_a) {
                                    if (item.serverUrl) {
                                        item.serverUrl = item.serverUrl.replace(/\/+$/, '');
                                    }
                                    return [2 /*return*/, item];
                                });
                            }); }))];
                    case 1:
                        _a.config = _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    Generator.prototype.generate = function () {
        return __awaiter(this, void 0, void 0, function () {
            var outputFileList;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        outputFileList = Object.create(null);
                        return [4 /*yield*/, Promise.all(this.config.map(function (serverConfig, serverIndex) { return __awaiter(_this, void 0, void 0, function () {
                                var projects;
                                var _this = this;
                                return __generator(this, function (_a) {
                                    projects = serverConfig.projects.reduce(function (projects, project) {
                                        projects.push.apply(projects, castArray(project.token).map(function (token) { return (__assign(__assign({}, project), { token: token })); }));
                                        return projects;
                                    }, []);
                                    return [2 /*return*/, Promise.all(projects.map(function (projectConfig, projectIndex) { return __awaiter(_this, void 0, void 0, function () {
                                            var projectInfo;
                                            var _this = this;
                                            return __generator(this, function (_a) {
                                                switch (_a.label) {
                                                    case 0: return [4 /*yield*/, this.fetchProjectInfo(__assign(__assign({}, serverConfig), projectConfig))];
                                                    case 1:
                                                        projectInfo = _a.sent();
                                                        return [4 /*yield*/, Promise.all(projectConfig.categories.map(function (categoryConfig, categoryIndex) { return __awaiter(_this, void 0, void 0, function () {
                                                                var categoryIds, excludedCategoryIds, codes, _i, _a, groupedCodes;
                                                                var _this = this;
                                                                return __generator(this, function (_b) {
                                                                    switch (_b.label) {
                                                                        case 0:
                                                                            categoryIds = castArray(categoryConfig.id);
                                                                            // 全部分类
                                                                            if (categoryIds.includes(0)) {
                                                                                categoryIds.push.apply(categoryIds, projectInfo.cats.map(function (cat) { return cat._id; }));
                                                                            }
                                                                            // 唯一化
                                                                            categoryIds = uniq(categoryIds);
                                                                            excludedCategoryIds = categoryIds
                                                                                .filter(function (id) { return id < 0; })
                                                                                .map(Math.abs);
                                                                            categoryIds = categoryIds.filter(function (id) { return !excludedCategoryIds.includes(Math.abs(id)); });
                                                                            // 删除不存在的分类
                                                                            categoryIds = categoryIds.filter(function (id) { return !!projectInfo.cats.find(function (cat) { return cat._id === id; }); });
                                                                            // 顺序化
                                                                            categoryIds = categoryIds.sort();
                                                                            return [4 /*yield*/, Promise.all(categoryIds.map(function (id, categoryIndex2) { return __awaiter(_this, void 0, void 0, function () {
                                                                                    var syntheticalConfig, interfaceList, outputFilePath, categoryCode, _a;
                                                                                    var _this = this;
                                                                                    return __generator(this, function (_b) {
                                                                                        switch (_b.label) {
                                                                                            case 0:
                                                                                                categoryConfig = __assign(__assign({}, categoryConfig), { id: id });
                                                                                                syntheticalConfig = __assign(__assign(__assign(__assign({}, serverConfig), projectConfig), categoryConfig), { mockUrl: projectInfo.getMockUrl() });
                                                                                                return [4 /*yield*/, this.fetchInterfaceList(syntheticalConfig)];
                                                                                            case 1:
                                                                                                interfaceList = (_b.sent())
                                                                                                    .map(function (interfaceInfo) {
                                                                                                    // 实现 _project 字段
                                                                                                    interfaceInfo._project = omit(projectInfo, [
                                                                                                        'cats',
                                                                                                        'getMockUrl',
                                                                                                    ]);
                                                                                                    return interfaceInfo;
                                                                                                })
                                                                                                    .sort(function (a, b) { return a._id - b._id; });
                                                                                                outputFilePath = path.resolve(this.options.cwd, syntheticalConfig.outputFilePath);
                                                                                                if (!(interfaceList.length === 0)) return [3 /*break*/, 2];
                                                                                                _a = '';
                                                                                                return [3 /*break*/, 4];
                                                                                            case 2: return [4 /*yield*/, Promise.all(interfaceList.map(function (interfaceInfo) { return __awaiter(_this, void 0, void 0, function () {
                                                                                                    return __generator(this, function (_a) {
                                                                                                        console.log('interfaceInfo', interfaceInfo);
                                                                                                        interfaceInfo = isFunction(syntheticalConfig.preproccessInterface)
                                                                                                            ? syntheticalConfig.preproccessInterface(interfaceInfo, changeCase)
                                                                                                            : interfaceInfo;
                                                                                                        return [2 /*return*/, {
                                                                                                                project_id: interfaceInfo.project_id,
                                                                                                                filePath: interfaceInfo.path,
                                                                                                                fileContent: this.generateInterfaceCode(syntheticalConfig, interfaceInfo),
                                                                                                                catid: interfaceInfo.catid
                                                                                                            }];
                                                                                                    });
                                                                                                }); }))];
                                                                                            case 3:
                                                                                                _a = __spreadArrays.apply(void 0, [(_b.sent())]);
                                                                                                _b.label = 4;
                                                                                            case 4:
                                                                                                categoryCode = _a;
                                                                                                console.log('categoryCode', categoryCode);
                                                                                                if (!outputFileList[outputFilePath]) {
                                                                                                    outputFileList[outputFilePath] = {
                                                                                                        syntheticalConfig: syntheticalConfig,
                                                                                                        content: [],
                                                                                                    };
                                                                                                }
                                                                                                return [2 /*return*/, {
                                                                                                        outputFilePath: outputFilePath,
                                                                                                        code: categoryCode ? __spreadArrays(categoryCode) : [],
                                                                                                        weights: [
                                                                                                            serverIndex,
                                                                                                            projectIndex,
                                                                                                            categoryIndex,
                                                                                                            categoryIndex2,
                                                                                                        ],
                                                                                                    }];
                                                                                        }
                                                                                    });
                                                                                }); }))];
                                                                        case 1:
                                                                            codes = _b.sent();
                                                                            for (_i = 0, _a = values(groupBy(codes, function (item) { return item.outputFilePath; })); _i < _a.length; _i++) {
                                                                                groupedCodes = _a[_i];
                                                                                groupedCodes.sort(function (a, b) {
                                                                                    var _a;
                                                                                    var x = a.weights.length > b.weights.length ? b : a;
                                                                                    var minLen = Math.min(a.weights.length, b.weights.length);
                                                                                    var maxLen = Math.max(a.weights.length, b.weights.length);
                                                                                    (_a = x.weights).push.apply(_a, new Array(maxLen - minLen).fill(0));
                                                                                    var w = a.weights.reduce(function (w, _, i) {
                                                                                        if (w === 0) {
                                                                                            w = a.weights[i] - b.weights[i];
                                                                                        }
                                                                                        return w;
                                                                                    }, 0);
                                                                                    return w;
                                                                                });
                                                                                console.log('groupedCodes', groupedCodes);
                                                                                outputFileList[groupedCodes[0].outputFilePath].content = __spreadArrays(groupedCodes.map(function (item) {
                                                                                    return item.code;
                                                                                }));
                                                                                console.log('outputFileList', outputFileList);
                                                                            }
                                                                            return [2 /*return*/];
                                                                    }
                                                                });
                                                            }); }))];
                                                    case 2:
                                                        _a.sent();
                                                        return [2 /*return*/];
                                                }
                                            });
                                        }); }))];
                                });
                            }); }))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/, outputFileList];
                }
            });
        });
    };
    Generator.prototype.write = function (outputFileList) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, Promise.all(Object.keys(outputFileList).map(function (outputFilePath) { return __awaiter(_this, void 0, void 0, function () {
                        var content, newContent;
                        var _this = this;
                        return __generator(this, function (_a) {
                            content = outputFileList[outputFilePath].content;
                            console.log('content', content);
                            newContent = [];
                            content.forEach(function (item) { return newContent.push.apply(newContent, item); });
                            console.log('newContent', values(groupBy(newContent, 'catid')));
                            values(groupBy(newContent, 'catid')).map(function (item) { return __awaiter(_this, void 0, void 0, function () {
                                var path, fliePath, fileContent, rawOutputContent, prettyOutputContent;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            path = item[0].filePath && item[0].filePath.split('/')[1];
                                            fliePath = ("src/" + path + "/" + path + "Api.ts").replace(/\.js(x)?$/, '.ts$1');
                                            fileContent = [];
                                            item && item.forEach(function (element) {
                                                fileContent.push(element.fileContent);
                                            });
                                            // 始终写入主文件
                                            console.log('fileContent11111', fileContent);
                                            rawOutputContent = dedent(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n            import { AdminPath, buildApiUrl } from '@/modules/Common/constants/ApiPath';\n            import Res from '@/modules/Common/utils/Res';\n            import { post } from '@/modules/Common/utils/request';\n            export default class ", "Api {\n            }\n          "], ["\n            import { AdminPath, buildApiUrl } from '@/modules/Common/constants/ApiPath';\n            import Res from '@/modules/Common/utils/Res';\n            import { post } from '@/modules/Common/utils/request';\n            export default class ", "Api {\n            }\n          "])), path);
                                            prettyOutputContent = prettier.format(rawOutputContent, {
                                                parser: 'typescript',
                                                printWidth: 120,
                                                tabWidth: 2,
                                                singleQuote: true,
                                                semi: false,
                                                trailingComma: 'all',
                                                bracketSpacing: false,
                                                endOfLine: 'lf',
                                            });
                                            return [4 /*yield*/, fs.outputFile(fliePath, prettyOutputContent)];
                                        case 1:
                                            _a.sent();
                                            return [2 /*return*/];
                                    }
                                });
                            }); });
                            return [2 /*return*/];
                        });
                    }); }))];
            });
        });
    };
    Generator.prototype.fetchApi = function (url, query) {
        return __awaiter(this, void 0, void 0, function () {
            var res;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, got.get(url, {
                            searchParams: query,
                            responseType: 'json',
                        })
                        /* istanbul ignore next */
                    ];
                    case 1:
                        res = (_a.sent()).body;
                        /* istanbul ignore next */
                        if (res && res.errcode) {
                            throwError(res.errmsg);
                        }
                        return [2 /*return*/, res.data || res];
                }
            });
        });
    };
    /** 获取分类的接口列表 */
    Generator.prototype.fetchInterfaceList = function (_a) {
        var serverUrl = _a.serverUrl, token = _a.token, id = _a.id;
        return __awaiter(this, void 0, void 0, function () {
            var category;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4 /*yield*/, this.fetchExport({ serverUrl: serverUrl, token: token })];
                    case 1:
                        category = ((_b.sent()) || []).find(function (cat) { return !isEmpty(cat) && !isEmpty(cat.list) && cat.list[0].catid === id; });
                        if (category) {
                            category.list.forEach(function (interfaceInfo) {
                                // 实现 _category 字段
                                interfaceInfo._category = omit(category, ['list']);
                            });
                        }
                        return [2 /*return*/, category ? category.list : []];
                }
            });
        });
    };
    /** 获取项目信息 */
    Generator.prototype.fetchProjectInfo = function (syntheticalConfig) {
        return __awaiter(this, void 0, void 0, function () {
            var projectInfo, projectCats;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.fetchProject(syntheticalConfig)];
                    case 1:
                        projectInfo = _a.sent();
                        return [4 /*yield*/, this.fetchApi(syntheticalConfig.serverUrl + "/api/interface/getCatMenu", {
                                token: syntheticalConfig.token,
                                project_id: projectInfo._id,
                            })];
                    case 2:
                        projectCats = _a.sent();
                        return [2 /*return*/, __assign(__assign({}, projectInfo), { cats: projectCats, getMockUrl: function () {
                                    return syntheticalConfig.serverUrl + "/mock/" + projectInfo._id;
                                } })];
                }
            });
        });
    };
    /** 生成接口代码 */
    Generator.prototype.generateInterfaceCode = function (syntheticalConfig, interfaceInfo) {
        return __awaiter(this, void 0, void 0, function () {
            var extendedInterfaceInfo, requestFunctionName, _a, requestDataTypeName, _b, responseDataTypeName, _c, escapedTitle;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        extendedInterfaceInfo = __assign(__assign({}, interfaceInfo), { parsedPath: path.parse(interfaceInfo.path) });
                        if (!isFunction(syntheticalConfig.getRequestFunctionName)) return [3 /*break*/, 2];
                        return [4 /*yield*/, syntheticalConfig.getRequestFunctionName(extendedInterfaceInfo, changeCase)];
                    case 1:
                        _a = _d.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _a = changeCase.camelCase(extendedInterfaceInfo.parsedPath.name);
                        _d.label = 3;
                    case 3:
                        requestFunctionName = _a;
                        if (!isFunction(syntheticalConfig.getRequestDataTypeName)) return [3 /*break*/, 5];
                        return [4 /*yield*/, syntheticalConfig.getRequestDataTypeName(extendedInterfaceInfo, changeCase)];
                    case 4:
                        _b = _d.sent();
                        return [3 /*break*/, 6];
                    case 5:
                        _b = changeCase.pascalCase(requestFunctionName + "Request");
                        _d.label = 6;
                    case 6:
                        requestDataTypeName = _b;
                        if (!isFunction(syntheticalConfig.getResponseDataTypeName)) return [3 /*break*/, 8];
                        return [4 /*yield*/, syntheticalConfig.getResponseDataTypeName(extendedInterfaceInfo, changeCase)];
                    case 7:
                        _c = _d.sent();
                        return [3 /*break*/, 9];
                    case 8:
                        _c = changeCase.pascalCase(requestFunctionName + "Response");
                        _d.label = 9;
                    case 9:
                        responseDataTypeName = _c;
                        escapedTitle = String(extendedInterfaceInfo.title).replace(/\//g, '\\/');
                        return [2 /*return*/, dedent(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n      /**\n       * \u63A5\u53E3 ", " \u7684 **\u8BF7\u6C42\u51FD\u6570**\n       *\n      */\n      public static ", "(body: ", ") {\n        return ", "<", ">(buildUrl('", "'), body);\n      }\n    "], ["\n      /**\n       * \u63A5\u53E3 ", " \u7684 **\u8BF7\u6C42\u51FD\u6570**\n       *\n      */\n      public static ", "(body: ", ") {\n        return ", "<", ">(buildUrl('", "'), body);\n      }\n    "])), escapedTitle, requestFunctionName, requestDataTypeName, extendedInterfaceInfo.method.toLowerCase(), responseDataTypeName, extendedInterfaceInfo.path)];
                }
            });
        });
    };
    /** 生成参数代码 */
    Generator.prototype.generateReqCode = function (syntheticalConfig, interfaceInfo) {
        return __awaiter(this, void 0, void 0, function () {
            var extendedInterfaceInfo, requestFunctionName, _a, requestDataTypeName, _b, responseDataTypeName, _c, requestDataJsonSchema, requestDataType, responseDataJsonSchema, responseDataType;
            return __generator(this, function (_d) {
                switch (_d.label) {
                    case 0:
                        extendedInterfaceInfo = __assign(__assign({}, interfaceInfo), { parsedPath: path.parse(interfaceInfo.path) });
                        if (!isFunction(syntheticalConfig.getRequestFunctionName)) return [3 /*break*/, 2];
                        return [4 /*yield*/, syntheticalConfig.getRequestFunctionName(extendedInterfaceInfo, changeCase)];
                    case 1:
                        _a = _d.sent();
                        return [3 /*break*/, 3];
                    case 2:
                        _a = changeCase.camelCase(extendedInterfaceInfo.parsedPath.name);
                        _d.label = 3;
                    case 3:
                        requestFunctionName = _a;
                        if (!isFunction(syntheticalConfig.getRequestDataTypeName)) return [3 /*break*/, 5];
                        return [4 /*yield*/, syntheticalConfig.getRequestDataTypeName(extendedInterfaceInfo, changeCase)];
                    case 4:
                        _b = _d.sent();
                        return [3 /*break*/, 6];
                    case 5:
                        _b = changeCase.pascalCase(requestFunctionName + "Request");
                        _d.label = 6;
                    case 6:
                        requestDataTypeName = _b;
                        if (!isFunction(syntheticalConfig.getResponseDataTypeName)) return [3 /*break*/, 8];
                        return [4 /*yield*/, syntheticalConfig.getResponseDataTypeName(extendedInterfaceInfo, changeCase)];
                    case 7:
                        _c = _d.sent();
                        return [3 /*break*/, 9];
                    case 8:
                        _c = changeCase.pascalCase(requestFunctionName + "Response");
                        _d.label = 9;
                    case 9:
                        responseDataTypeName = _c;
                        requestDataJsonSchema = getRequestDataJsonSchema(extendedInterfaceInfo);
                        return [4 /*yield*/, jsonSchemaToType(requestDataJsonSchema, requestDataTypeName)];
                    case 10:
                        requestDataType = _d.sent();
                        responseDataJsonSchema = getResponseDataJsonSchema(extendedInterfaceInfo, syntheticalConfig.dataKey);
                        return [4 /*yield*/, jsonSchemaToType(responseDataJsonSchema, responseDataTypeName)];
                    case 11:
                        responseDataType = _d.sent();
                        return [2 /*return*/, dedent(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n      ", "\n      ", "\n    "], ["\n      ", "\n      ", "\n    "])), responseDataType.trim(), requestDataType.trim())];
                }
            });
        });
    };
    Generator.prototype.destroy = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, Promise.all(this.disposes.map(function (dispose) { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                        return [2 /*return*/, dispose()];
                    }); }); }))];
            });
        });
    };
    return Generator;
}());
export { Generator };
var templateObject_1, templateObject_2, templateObject_3;
